
' ParaChuters v1.0
' (C) 2007 i-TECH and Kristopher Windsor

#define use_framerate
#define use_highscores
#define use_iinput
#define use_imci
#define use_particles
#include "itech\main.bi"

Const screenx = 480, screeny = 320

Const uairplane = 2, ubomb = 32, ucannon = 1, uhelicopter = 32, umissile = 32, utrooper = 512

Const airplane_size = 20, bomb_size = 4, helicopter_size = 15, missile_size = 8, trooper_size = 10
Const trooper_width = 5
Const cannon_height = 100, cannon_width_half = 24

Const helicopter_flight_location_y = 40, airplane_flight_location_y = 20
Const cannon_powermissile_total = 4

Const white = &HFFFFFFFF, green = &HFF55FF55, blue = &HFF5555FF, brown = &HFF906010, black = &HFF000000, gray = &HFF333333

Type airplane
  As Double x, v
  As Integer killme
End Type

Type bomb
  As Double x, y, xv, yv
  As Integer killme
End Type

Type cannon
  As Double x, angle
  As Integer powermissiles
  As Integer killme
End Type

Type helicopter
  As Double x, v, a
  As Double dropspot
  As Integer killme
End Type

Type missile
  As Double x, y, xv, yv
  As Double power
  As Integer killme
End Type

Type trooper
  As Double x, y, v
  As Integer landed
  As Integer killme
End Type

Dim Shared As Integer airplane_total, bomb_total, cannon_total, helicopter_total, missile_total, trooper_total
Dim Shared As Integer score, powermissiles_shouldfire, quit, lost, game_over, lost_extra_frames
Dim Shared As Double wind_velocity
Dim Shared As airplane airplanes(1 To uairplane)
Dim Shared As bomb bombs(1 To ubomb)
Dim Shared As cannon cannons(1 To ucannon)
Dim Shared As helicopter helicopters(1 To uhelicopter)
Dim Shared As missile missiles(1 To umissile)
Dim Shared As trooper troopers(1 To utrooper)
Dim Shared As Uinteger Ptr background, eraser

Declare Sub bomb_add (Byref x As Double, Byref y As Double, Byref xv As Double, Byref yv As Double)
Declare Sub missile_add (Byref x As Double, Byref y As Double, Byref angle As Double, Byref power As Double)
Declare Sub trooper_add (Byref x As Double, Byref y As Double)

Sub airplane_reset
  airplane_total = 0
End Sub

Sub airplane_add (Byval direction As Integer)
  If airplane_total = uairplane Then Exit Sub
  direction = Sgn(direction): If direction = 0 Then direction = -1 'allows either flagset: -1, 1 or 0, 1
  airplane_total += 1
  With airplanes(airplane_total)
    Select Case direction
    Case -1: .x = screenx + 49
    Case 1: .x = -50
    End Select
    .v = (Rnd * 3 + 1) * direction
    .killme = false
  End With
End Sub

Sub airplane_move
  Dim As Integer a

  For a = 1 To airplane_total
    With airplanes(a)
      .x += .v
      If .x < -50 Or .x >= 50 + screenx Then .killme = true

      'add bomb
      If framerate_frame Mod 10 = 0 Then
        For b As Integer = 1 To cannon_total
          If Abs(.x - cannons(b).x) < cannon_width_half Then bomb_add .x, airplane_flight_location_y, 0, 4
        Next b
      End If
    End With
  Next a

  a = 1
  While a <= airplane_total And airplane_total > 0
    If airplanes(a).killme Then
      For b As Integer = 1 To cannon_total
        With cannons(b)
          If .powermissiles < cannon_powermissile_total Then .powermissiles += 1
        End With
      Next b
      Swap airplanes(a), airplanes(airplane_total)
      airplane_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub airplane_display
  Dim As Integer s
  For a As Integer = 1 To airplane_total
    With airplanes(a)
      s = Sgn(.v)
      'floor and ceiling
      Line (.x - 13, airplane_flight_location_y - 6) - Step(26, 0), black
      Line (.x - 13, airplane_flight_location_y + 6) - Step(26, 0), black
      Line (.x - 13 * s, airplane_flight_location_y + 6) - Step(-5 * s, 0), black
      'front
      Circle (.x + 14 * s, airplane_flight_location_y), 6, black, pi_half - pi * (s = 1), pi_half - pi * (s = -1), 1
      'back
      Line (.x - 13 * s, airplane_flight_location_y - 6) - Step(-12 * s, -12), black
      Line (.x - 18 * s, airplane_flight_location_y + 6) - Step(-7 * s, -24), black
      'short fin
      Line (.x - 4, airplane_flight_location_y - 6) - (.x, airplane_flight_location_y - 15), black
      Line (.x + 4, airplane_flight_location_y - 6) - (.x, airplane_flight_location_y - 15), black
      'long fin
      Line (.x - 5, airplane_flight_location_y + 6) - (.x, airplane_flight_location_y + 20), black
      Line (.x + 5, airplane_flight_location_y + 6) - (.x, airplane_flight_location_y + 20), black
    End With
  Next a
End Sub

Sub bomb_reset
  bomb_total = 0
End Sub

Sub bomb_add (Byref x As Double, Byref y As Double, Byref xv As Double, Byref yv As Double)
  If bomb_total = ubomb Then Exit Sub
  bomb_total += 1
  With bombs(bomb_total)
    .x = x
    .y = y
    .xv = xv
    .yv = yv
    .killme = false
  End With
End Sub

Sub bomb_move
  Dim As Integer a

  For a = 1 To bomb_total
    With bombs(a)
      .x += .xv
      .y += .yv
      If .x < 0 Or .x > screenx - 1 Or .y < 0 Or .y > screeny - 1 Then .killme = true
      'collision with cannons
      For b As Integer = 1 To cannon_total
        If Abs(.x - cannons(b).x) < cannon_width_half + bomb_size And .y > screeny - cannon_height - cannon_width_half Then
          .killme = true: cannons(b).killme = true
          For c As Integer = -20 To 20 Step 5
            For d As Integer = -20 To 20 Step 5
              particle_add cannons(b).x + c, screeny - cannon_height + d
            Next d
          Next c
        End If
      Next b
    End With
  Next a

  a = 1
  While a <= bomb_total And bomb_total > 0
    If bombs(a).killme Then
      Swap bombs(a), bombs(bomb_total)
      bomb_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub bomb_display
  For a As Integer = 1 To bomb_total
    With bombs(a)
      Circle (.x, .y), bomb_size, black,,, 1, f
      Circle (.x, .y), bomb_size - 1, gray,,, 1, f
    End With
  Next a
End Sub

Sub cannon_reset
  cannon_total = 0
End Sub

Sub cannon_add
  If cannon_total = ucannon Then Exit Sub
  cannon_total += 1
  With cannons(cannon_total)
    .x = screenx * .5
    .angle = -pi_half
    .powermissiles = cannon_powermissile_total
    .killme = false
  End With
End Sub

Sub cannon_move
  Dim As Integer a
  Dim As Double d, power

  For a = 1 To cannon_total
    With cannons(a)
      'angle defined by mouse
      d = Atan2(my + cannon_height - screeny, mx - .x)
      'slowly apply angle
      If d < 0 Then
        Select Case .angle - d
        Case Is < -.1
          .angle += .1
        Case Is > .1
          .angle -= .1
        Case Else
          .angle = d
        End Select
      End If
      'add missile
      If framerate_frame Mod 10 = 0 Then
        power = 8
        If powermissiles_shouldfire And .powermissiles > 0 Then power = 120: .powermissiles -= 1
        missile_add .x, screeny - cannon_height, .angle, power
        powermissiles_shouldfire = false
      End If
    End With
  Next a

  a = 1
  While a <= cannon_total And cannon_total > 0
    If cannons(a).killme Then
      Swap cannons(a), cannons(cannon_total)
      cannon_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub cannon_display
  For a As Integer = 1 To cannon_total
    With cannons(a)
      Line (.x - cannon_width_half, screeny - cannon_height) - (.x + cannon_width_half, screeny - 1), brown, bf
      Circle (.x, screeny - cannon_height), cannon_width_half, brown,,,, f
      Line Step(0, 0) - Step(Cos(.angle) * cannon_width_half, Sin(.angle) * cannon_width_half), black
      For b As Integer = 1 To .powermissiles
        Line (.x - cannon_width_half / 2, screeny - b * 5) - Step(cannon_width_half, 3), black, bf
      Next b
    End With
  Next a
End Sub

Sub helicopter_reset
  helicopter_total = 0
End Sub

Sub helicopter_add (Byval direction As Integer)
  If helicopter_total = uhelicopter Then Exit Sub
  direction = Sgn(direction): If direction = 0 Then direction = -1 'allows either flagset: -1, 1 or 0, 1
  helicopter_total += 1
  With helicopters(helicopter_total)
    Select Case direction
    Case -1: .x = screenx + 49
    Case 1: .x = -50
    End Select
    .v = (Rnd * 3 + 1) * direction
    .dropspot = Int(Rnd * screenx)
    .a = 0
    .killme = false
  End With
End Sub

Sub helicopter_move
  Dim As Integer a

  For a = 1 To helicopter_total
    With helicopters(a)
      .x += .v
      .a += .1
      If .a > pi_double Then .a -= pi_double
      If .x < -50 Or .x >= 50 + screenx Then .killme = true
      'add trooper
      If .dropspot > 0 And Abs(.x - .dropspot) < 5 Then trooper_add .x, helicopter_flight_location_y: .dropspot = -1
    End With
  Next a

  a = 1
  While a <= helicopter_total And helicopter_total > 0
    If helicopters(a).killme Then
      Swap helicopters(a), helicopters(helicopter_total)
      helicopter_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub helicopter_display
  Dim As Double c

  For a As Integer = 1 To helicopter_total
    With helicopters(a)
      'helicopter display code ported from my QBasic game, Bounce 3: On the Town
      Line (.x - 9, helicopter_flight_location_y - 6)-(.x + 9, helicopter_flight_location_y + 6), blue, b
      If .v < 0 Then
       Circle (.x - 10, helicopter_flight_location_y), 6, blue, pi_half, 3 * pi_half, 1
       Line (.x + 9, helicopter_flight_location_y - 6)-Step(16, 3), blue
       Line Step(0, 0)-Step(-16, 9), blue
       Line Step(-15, 0)-Step(0, 3), blue: Line Step(-4, 0)-Step(8, 0), blue
       Line Step(5, -3)-Step(0, 5), blue: Line Step(-4, 0)-Step(8, 0), blue
       For v As Double = 0 To pi_double Step pi_quarter
        c = .a + v: If c > pi_double Then c = c - pi_double
        If c > pi Then Line (.x, helicopter_flight_location_y - 6)-Step(Cos(c) * 14, Sin(c) * 8), blue
        c = .a * 2 + v: Line (.x + 25, helicopter_flight_location_y - 3)-Step(Cos(c) * 5, Sin(c) * 5), blue
       Next v
      Else
       Circle (.x + 10, helicopter_flight_location_y), 6, blue, 3 * pi_half, pi_half, 1
       Line (.x - 9, helicopter_flight_location_y - 6)-Step(-16, 3), blue
       Line Step(0, 0)-Step(16, 9), blue
       Line Step(15, 0)-Step(0, 3), blue: Line Step(4, 0)-Step(-8, 0), blue
       Line Step(-5, -3)-Step(0, 5), blue: Line Step(4, 0)-Step(-8, 0), blue
       For v As Double = 0 To pi_double Step pi_quarter
        c = .a + v: If c > pi_double Then c = c - pi_double
        If c > pi Then Line (.x, helicopter_flight_location_y - 6)-Step(Cos(c) * 14, Sin(c) * 8), blue
        c = .a * 2 + v: Line (.x - 25, helicopter_flight_location_y - 3)-Step(Cos(c) * 5, Sin(c) * 5), blue
       Next v
      End If
    End With
  Next a
End Sub

Sub missile_reset
  missile_total = 0
End Sub

Sub missile_add (Byref x As Double, Byref y As Double, Byref angle As Double, Byref power As Double)
  If missile_total = umissile Then Exit Sub
  missile_total += 1
  With missiles(missile_total)
    .xv = Cos(angle) * 8
    .yv = Sin(angle) * 8
    .x = x + .xv * 2
    .y = y + .yv * 2
    .power = power
    .killme = false
  End With
End Sub

Sub missile_move
  Dim As Integer a, killflag
  Dim As Double x, y, d

  For a = 1 To missile_total
    With missiles(a)
      .x += .xv
      .y += .yv
      If .x < -160 Or .x > screenx + 159 Or .y < -160 Then .killme = true
      'add particle trail for power missiles
      If .power > 10 And framerate_frame Mod 4 = 0 Then particle_add .x, .y
      'when a missile explodes, it destroys everything next to it
      'so when a missiles explodes, its range increases, and every collision has to be rechecked
      'the kill flag will alows the game to recheck collision if the missile exploded in the first run of the loop
      'if a missile is a power missile, it has to hit something, then its range increase, and everything in site explodes
      Do
        killflag = .killme
        'collide with airplanes
        For b As Integer = 1 To airplane_total
          If airplanes(b).killme = false Then
            x = .x - airplanes(b).x
            y = .y - airplane_flight_location_y
            d = missile_size + airplane_size
            If .killme Then d += .power
            If x * x + y * y < d * d Then
              score += 25
              .killme = true
              airplanes(b).killme = true
              particle_add airplanes(b).x, airplane_flight_location_y
            End If
          End If
        Next b
        'collide with helicopters
        For b As Integer = 1 To helicopter_total
          If helicopters(b).killme = false Then
            x = .x - helicopters(b).x
            y = .y - helicopter_flight_location_y
            d = missile_size + helicopter_size
            If .killme Then d += .power
            If x * x + y * y < d * d Then
              score += 5
              .killme = true
              helicopters(b).killme = true
              particle_add helicopters(b).x, helicopter_flight_location_y
            End If
          End If
        Next b
        'collide with troopers
        For b As Integer = 1 To trooper_total
          If troopers(b).killme = false Then
            x = .x - troopers(b).x
            y = .y - troopers(b).y
            d = missile_size + trooper_size
            If .killme Then d += .power
            If x * x + y * y < d * d Then
              score += 1
              .killme = true
              troopers(b).killme = true
              particle_add troopers(b).x, troopers(b).y
            End If
          End If
        Next b
      Loop While killflag = false And .killme = true
    End With
  Next a

  a = 1
  While a <= missile_total And missile_total > 0
    If missiles(a).killme Then
      Swap missiles(a), missiles(missile_total)
      missile_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub missile_display
  For a As Integer = 1 To missile_total
    With missiles(a)
      Line (.x, .y) - Step(-.xv, -.yv), white
    End With
  Next a
End Sub

Sub trooper_reset
  trooper_total = 0
End Sub

Sub trooper_add (Byref x As Double, Byref y As Double)
  If trooper_total = utrooper Then Exit Sub
  trooper_total += 1
  With troopers(trooper_total)
    .x = x + Rnd 'rnd stops troopers from sharing the same height
    .y = y
    .v = 3 * Rnd + 1
    .landed = false
    .killme = false
  End With
End Sub

Sub trooper_move
  Dim As Integer a, issupported

  For a = 1 To trooper_total
    With troopers(a)
      If .landed Then
        'optimal position is no overlapping troopers
        'but there should be another trooper below every trooper in the air

        'allign column of troopers
        For b As Integer = 1 To trooper_total
          If a <> b And troopers(b).landed And troopers(b).y >= .y Then
            If Abs(.x - troopers(b).x) < trooper_width * 2 And Abs(.y - troopers(b).y) < trooper_size * 2 Then
              If Abs(troopers(b).x - .x) < .2 Then
                .y -= Rnd / 8
              Else
                .x += Sgn(troopers(b).x - .x) * Rnd / 8
              End If
            End If
          End If
        Next b

        'fall if no longer supported
        If .y < screeny - trooper_size Then
          issupported = false
          For b As Integer = 1 To trooper_total
            If troopers(b).landed Then
              If a <> b And troopers(b).y > .y And Abs(.x - troopers(b).x) < trooper_width * 2 And Abs(.y - troopers(b).y) < trooper_size * 2 + 2 Then issupported = true
            End If
          Next b
          If issupported = false Then .landed = false
        End If

        'make bomb (instant game over) if supported at high altitude
        If framerate_frame Mod 10 = 0 Then
          If .landed And .y < screeny - cannon_height Then
            bomb_add .x, .y, Sgn(cannons(1).x - .x) * 4, Sgn(screeny - cannon_height - .y) * 4 * (Abs(screeny - cannon_height - .y) / Abs(cannons(1).x - .x))
          End If
        End If
      Else
        .x += wind_velocity
        If .x < 0 Or .x > screenx - 1 Then .killme = true
        .y += .v
        If .y >= screeny - trooper_size Then .landed = true

        'land on other troopers
        For b As Integer = 1 To trooper_total
          If troopers(b).landed Then
            If Abs(.x - troopers(b).x) < trooper_width * 2 And Abs(.y - troopers(b).y) < trooper_size * 2 Then .landed = true
          End If
        Next b

        'kill if touching cannon
        For b As Integer = 1 To cannon_total
          If Abs(.x - cannons(b).x) < cannon_width_half + trooper_width And .y > screeny - cannon_height - cannon_width_half Then .killme = true
        Next b
      End If
    End With
  Next a

  a = 1
  While a <= trooper_total And trooper_total > 0
    If troopers(a).killme Then
      Swap troopers(a), troopers(trooper_total)
      trooper_total -= 1
    Else
      a += 1
    End If
  Wend
End Sub

Sub trooper_display
  For a As Integer = 1 To trooper_total
    With troopers(a)
      'head
      Circle (.x, .y - 6), 4, green,,, 1
      Pset (.x - 1, .y - 7), green
      Pset (.x + 1, .y - 7), green
      'arms
      Line (.x - 5, .y + 1) - Step(10, 0), green
      'chest
      Line (.x, .y - 1) - Step(0, 3), green
      'legs
      Line (.x, .y + 3) - Step(-4, 6), green
      Line (.x, .y + 3) - Step(4, 6), green
      'parachute
      If .landed = false Then
        Line (.x, .y) - Step(-9, -11), green
        Line (.x, .y) - Step(9, -11), green
        Line (.x - 10, .y - 11) - Step(20, 0), green
        Circle (.x, .y - 11), 10, green, 0, pi, 1
      End If
    End With
  Next a
End Sub

Sub computer_run
  'control wind
  If Int(Rnd * framerate_rate) = 0 Then
    wind_velocity += (Rnd - Rnd) * .2
    If wind_velocity < -2 Then wind_velocity = -2
    If wind_velocity > 2 Then wind_velocity = 2
  End If

  'add airplane
  If Int(Rnd * framerate_rate * (60 - score / 12)) = 0 Or score > 720 Then airplane_add Sgn(Rnd - Rnd)

  'add helicopter
  If Int(Rnd * (80 - score / 12)) = 0 Or score > 960 Then
    helicopter_add Sgn(Rnd - Rnd)
  End If
End Sub

Sub input_process
  updatemouse
  updatekey
  simulatemouse
  If (mb > 0 And pmb = 0) Then powermissiles_shouldfire = true
  If key = Chr(27) Then quit = (imenu("Quit Game?", "No:::Yes""") = 2) Or quit
  If iskeymousing And game_over Then quit = true
End Sub

Sub determine_winloss
  If cannon_total = 0 Then lost = true
  If lost Then lost_extra_frames += 1
  If lost_extra_frames > framerate_rate * 2 Then game_over = true
End Sub

Sub help_rules
  Do
    Screenlock
    Put (0, 0), eraser, alpha, 20
    font_display " [ParaChuters Rules] \ Aim the cannon to destroy \ the enemies! \ If the troopers pile up \ high enough they will \ destroy you! \ " & _
      "The airplane bombs will \ instantly destroy you! \ Click to fire your power \ missiles! ", 4, 4,, true
    Screenunlock

    Sleep 50
    updatemouse: updatekey
  Loop Until iskeymousing
End Sub

Sub help_scoring
  Do
    Screenlock
    Put (0, 0), eraser, alpha, 20
    font_display " [ParaChuters Scoring] \ Destroy a ParaChuter \ [1 Point] \ Destroy a Helicopter \ [5 points] \ Destroy an Airplane \ [25 points] ", 4, 4,, true
    Screenunlock

    Sleep 50
    updatemouse: updatekey
  Loop Until iskeymousing
End Sub

Sub game_start
  start screenx, screeny
  imci_play "data\open.mid"
  font_start "data\font.bmp"
  highscore_start "data\ParaChuters Scores.dat"
  particle_start 2, pi / 16
  background = imagecreate(screenx, screeny)
  eraser = imagecreate(screenx, screeny, &HFF000000)
  Bload "data\background.bmp", background
End Sub

Sub game_reset
  particle_reset
  airplane_reset
  bomb_reset
  cannon_reset
  helicopter_reset
  missile_reset
  trooper_reset
  cannon_add
  score = 0
  powermissiles_shouldfire = 0
  wind_velocity = 0
  quit = false
  lost = false
  lost_extra_frames = 0
  game_over = 0 'if lost two seconds ago
  framerate_set 30
End Sub

Sub game_move
  computer_run
  input_process

  particle_move

  airplane_move
  bomb_move
  cannon_move
  helicopter_move
  missile_move
  trooper_move

  determine_winloss
End Sub

Sub game_display
  Screenlock
  Put (0, 0), background, alpha, 80
  Put (0, 0), eraser, alpha, Iif(40 + lost_extra_frames > 255, 255, 40 + lost_extra_frames) 'blackness keeps trails from burning onto screen
  font_display " Score " & score & " ", font_center, 4,, true
  If game_over Then font_display " Game Over ", font_center, screeny * .4,, true

  particle_display

  airplane_display
  bomb_display
  cannon_display
  helicopter_display
  missile_display
  trooper_display
  Screenunlock
End Sub

Sub game_play
  imci_play "data\music.mid"
  Do
    framerate_timer += imci_repeat(165)
    game_move
    game_display
    framerate_keep
  Loop Until quit
End Sub

Sub game_finish
  finish
  imagedestroy(eraser)
End Sub

Dim As Integer can_start

game_start

Do
  Select Case imenu("ParaChuters!", "Play:::Rules:::Scoring")
  Case 1: can_start = true
  Case 2: help_rules
  Case 3: help_scoring
  End Select
Loop Until can_start

Do
  game_reset
  game_play
  If highscore_add(score) Then imci_play "data\won.mid" Else imci_play "data\lost.mid"
  highscore_display
Loop Until imenu("Play Again?", "Yes:::No") = 2
game_finish
System
